function [data_orig,X] = ssbEuler(sysInfo,fmDataInfo)     
% % split-step stochastic backward Euler: 
% ref: Mattingly, Stuart, Higham 2002: Equation (6.4)
%    Ergodicity for SDEs and approximations: locally Lipschitz vector fields and degenerate noise
%{
 solve the system
     dX = f(X)dt + sigma*dW
 by the split backward Euler: (an implicit scheme)
     X_*    = X(n) + dt* f(X_*);          
     X(n+1) = X_* + sigma* dWdt 
% with X =  and dWdt = sigma*sqrt(dt) * N(0,1)
% Output
%    data_orig  
%}

% f       = ssmInfo.gradV; 
% sigma   = diag(sysInfo.sigma_1,sysInfo.sigma_2); 

dt    = fmDataInfo.dt;  
tN    = fmDataInfo.tN;
N     = tN;
Newton_maxIte   = fmDataInfo.Newton_maxIte;
Newton_stopCond = fmDataInfo.Newton_stopCond;
if nargin <3
    X_init =   fmDataInfo.getInitial(1);
end

% Initialize the positions of (x, y)
X      =   zeros(2,tN); X(:,1) =   X_init;

% At each time grid, record the largetst (max_eigenvalue) and the smallest (min_eigenvalue) of the Newton iteration matrix 
% used in each implicit Euler step
% Eig_Newton(1,:)= max (eig) among one complete Newton solver; Eig_Newton(2,:)=min (eig) among one complete Newton solver; 
% the LAST time grid i=tN is not computed;
Eig_Newton = zeros(2,tN);

% Assign thermal noises
dWdt   =   sqrt(dt)*[sigma_x; sigma_y].*randn(2,tN);  % random_noise: i.i.d increments of sigma* Bm for the integrator: 
                   % generate random noise sqrt(2/beta)dB(t), which is approximated by sqrt(2/beta)*sqrt(Deta t)*N(0,1) 


for i=2:N     % split Backward Euler to integrate (x,y), 
% Implement the first step of Backward Eulter method
% The implicit system is solved by Newton's method
    % initialize the Newton method: 
    ite      = 0;
    % current position of (x, y)
    x_curr   = X(1,i-1); y_curr   = X(2,i-1);
    x_newton = x_curr;   y_newton = y_curr;
    % compute the vector-valed function f(x,y) of the implicit system
    f1 = x_newton-x_curr+dt*dVx(x_newton, y_newton);
    f2 = y_newton-y_curr+dt*dVy(x_newton, y_newton);
    % compute the gradient of f(x,y)
    grad_f=zeros(2,2);
    grad_f(1,1)= 1+dt*(dVxx(x_newton, y_newton));
    grad_f(1,2)= dt*(dVxy(x_newton, y_newton));
    grad_f(2,1)= dt*(dVxy(x_newton, y_newton));
    grad_f(2,2)= 1+dt*(dVyy(x_newton, y_newton));
    f_res = 100; %max(abs([f1;f2]));
    
    grad_f_eig_max=0;
    grad_f_eig_min=100;
    
    %% compute the max and min eigenvalues of grad_f
    dVxx_temp=dVxx(x_newton, y_newton);dVyy_temp=dVyy(x_newton, y_newton); dVxy_temp=dVxy(x_newton, y_newton);
    %compute the max and min eigenvalues of grad_f using the quadratic roots formula    
    temp_b=2+dt*(dVxx_temp+dVyy_temp);
    %temp_c=1+dt*(dVxx_temp+dVyy_temp)+dt^2*(dVxx_temp*dVyy_temp-dVxy_temp^2);
    
    temp_grad_f_eig_max=((temp_b)+dt*sqrt((dVxx_temp-dVyy_temp)^2+4*dVxy_temp^2))/2;
    temp_grad_f_eig_min=((temp_b)-dt*sqrt((dVxx_temp-dVyy_temp)^2+4*dVxy_temp^2))/2;
    
    if grad_f_eig_max<temp_grad_f_eig_max
        grad_f_eig_max=temp_grad_f_eig_max;
    end
    
    if grad_f_eig_min>temp_grad_f_eig_min
        grad_f_eig_min=temp_grad_f_eig_min;
    end
    
    while (f_res> Newton_stopCond ) && (ite<= Newton_maxIte)
        
        ite=ite+1;
        temp=[x_newton;y_newton]-grad_f\[f1;f2];
        x_newton=temp(1); y_newton=temp(2);
        % compute the vector-valed function f(x,y) of the implicit system
        f1 = x_newton-x_curr+dt*dVx(x_newton, y_newton);
        f2 = y_newton-y_curr+dt*dVy(x_newton, y_newton);
        % compute the gradient of f(x,y), which is equal to grad_f:= eye(n=2)+dt*Hessian(x_newton, y_newton)
        %grad_f=zeros(2,2);
        %% grad_f=eye(2,2)+dt*hessain
        dVxx_temp=dVxx(x_newton, y_newton);dVyy_temp=dVyy(x_newton, y_newton); dVxy_temp=dVxy(x_newton, y_newton);
    
        grad_f(1,1)= 1+dt*( dVxx_temp);
        grad_f(1,2)= dt*dVxy_temp;
        grad_f(2,1)= dt*dVxy_temp;
        grad_f(2,2)= 1+dt*(dVyy_temp );
    

       
       %% Update the residual of the vector valued function
        f_res=max(abs([f1;f2]));
    end
       
    if (f_res> Newton_stopCond )
        fprintf('Newton method does not converge, t=%i \n', i);
    end
       
    %% Implement the second step of Backward Eulter method
    X(1,i) = x_newton+dWdt(1,i-1);
    X(2,i) = y_newton+dWdt(2,i-1);
    
    %% Store the eigenvalues of grad_f during the Newton's method
    Eig_Newton(1, i-1)=grad_f_eig_max;
    Eig_Newton(2, i-1)=grad_f_eig_min;
       
end

% Output data; 
data_orig = struct( ...
    'X',        X,...
    'dWdt',     dWdt, ...
    'tN',       tN, ...
    'dt',       dt, ...
    'x_range',  [min(X(1,:)), max(X(1,:))], ...
    'y_range',  [min(X(2,:)), max(X(2,:))], ...
    'Eig_Newton', Eig_Newton ... % Record the eigenvalues of Newton iteration matrix at each time grid, the last grid tN is not computed
);

end

